#!/usr/bin/perl

use Socket;

# manually implemented imap module for perl and cyrus imap server
# very primitive implementation here, just enough to do what we need
# this is cyrus specific, no general purpose imap parsing code is here.

# this is because the Cyrus IMAP.so perl module gives me a sigsegv.

# caller has to remember the socket.

# set LOG to 1 if you want protocol sent to stdout.

my $LOG = 0;
my $seq = 0;	# imap protocol sequence number

my $rbuf;	# receive buffer for protocol data

sub
msgimap_debug
{
	my ($log) = @_;

	$LOG = $log;
}

# Determine the IMAP separator character
sub
msgimap_sep
{
        my $sep='.';
        open(FD, "/etc/imapd.conf");
        while(<FD>) {
                if (/^unixhierarchysep/) {
                        if(/yes/) {
                                $sep = '/';
                        }
                }
        }
        close(FD);
        return $sep;
}

# Determine if we're running with everything under INBOX for users
sub
msgimap_altnamespace
{
        my $altnamespace=0;
        open(FD, "/etc/imapd.conf");
        while(<FD>) {
                if (/^altnamespace/) {
                        if(/yes/) {
                                $altnamespace = 1;
                        }
                }
        }
        close(FD);
        return $altnamespace;
}

# the higher level imap routines.

sub
msgimap_login
{
	my ($S, $user, $pass) = @_;

	my $ret = msgimap_protsend($S, "$seq LOGIN $user $pass\r\n",
		"$seq OK", "$seq NO", "$seq BAD");
	$seq++;
	return($ret);
}

sub
msgimap_createmailbox
{
	my ($S, $mbox) = @_;

	my $ret = msgimap_protsend($S, "$seq CREATE $mbox\r\n",
		"$seq OK", "$seq NO", "$seq BAD");
	$seq++;
	return($ret);
}

sub
msgimap_subscribemailbox
{
	my ($S, $mbox) = @_;

	my $ret = msgimap_protsend($S, "$seq SUBSCRIBE $mbox\r\n",
		"$seq OK", "$seq NO", "$seq BAD");
	$seq++;
	return($ret);
}

# logout and close the socket
sub
msgimap_logout
{
	my ($S) = @_;

	my $ret = msgimap_protsend($S, "$seq LOGOUT\r\n",
		"$seq OK", "$seq NO", "$seq BAD");
	$seq++;
	close($S);
}

# implement the imap LIST command
# list folders as specified, note that user.admin will be listed as INBOX
# when logged in as admin, as will user.inbox when logged in as a user.
# users can create their own sub folders and it will be listed as INBOX.sub.
# one can force a listing to be user.uid.* if you list folders under that user
# rather than *, example:
# msgimap_listmailbox(INBOX, '*') will return "INBOX" and "INBOX.sub"
# msgimap_listmailbox("user.uid", *) will return "user.uid" and "user.uid.sub"
# '*' logged in as admin will return user.uid.* for each user.
# second argument filter as '%' will return only an exact match for the mailbox.
# the INBOX = user.uid is a cyrus specific behavior.
sub
msgimap_listmailbox
{
	my ($S, $mailbox, $filter) = @_;
	my (@lines, $lines, $mbox, @tmp, @mailboxes);

	my $ret = msgimap_protsend($S, "$seq LIST $mailbox $filter\r\n",
		"$seq OK", "$seq NO", "$seq BAD");
	$seq++;
	if ($ret != 0) {
		return($ret);
	}
	# split into an array based on newlines
	@lines = split("\r\n", $rbuf);
	foreach $line (@lines) {
		if (substr($line, 0, 7) ne "* LIST ") {
			next;
		}
		@tmp = split("[ \"]", $line);
		$mbox = $tmp[scalar(@tmp) - 1];
		push(@mailboxes, $mbox);
	}
	return(@mailboxes);
}

# cyrus acl settings pass in mailbox, user who gets new acl, and the acl.
sub
msgimap_setaclmailbox
{
	my ($S, $mailbox, $user, $acl) = @_;

	my $ret = msgimap_protsend($S, "$seq SETACL $mailbox $user $acl\r\n",
		"$seq OK", "$seq NO", "$seq BAD");
	$seq++;
	if ($ret != 0) {
		return($ret);
	}
	return($ret);
}

# cyrus delete mailbox
sub
msgimap_deletemailbox
{
	my ($S, $mailbox) = @_;

	my $ret = msgimap_protsend($S, "$seq DELETE $mailbox\r\n",
		"$seq OK", "$seq NO", "$seq BAD");
	$seq++;
	if ($ret != 0) {
		return($ret);
	}
	return($ret);
}

# cyrus set quota
sub
msgimap_setquota
{
	my ($S, $mailbox, $quota) = @_;

	my $ret = msgimap_protsend($S, "$seq SETQUOTA $mailbox (STORAGE $quota)\r\n",
		"$seq OK", "$seq NO", "$seq BAD");
	$seq++;
	if ($ret != 0) {
		return($ret);
	}
	return($ret);
}

# cyrus get quota
sub
msgimap_getquota
{
	my ($S, $mailbox) = @_;
	my (@lines, $line, @tokens, @quota);

	my $ret = msgimap_protsend($S, "$seq GETQUOTAROOT $mailbox\r\n",
		"$seq OK", "$seq NO", "$seq BAD");
	$seq++;
	# NO response means no quota, this is ok, return empty list
	if ($ret == -2) {
		return();
	}
	if ($ret != 0) {
		return($ret);
	}
	# split into an array based on newlines
	@lines = split("\r\n", $rbuf);
	foreach $line (@lines) {
		if (substr($line, 0, 8) ne "* QUOTA ") {
			next;
		}
		@tokens = split("[( )]+", $line);
		@quota[0] = $tokens[4];
		@quota[1] = $tokens[5];
	}
	return(@quota);
}

# the low level imap routines, msgimap_connect is called externally
# but the rest are called internally only for now.

# connect returns a socket if all goes well.
sub
msgimap_connect
{
	my ($machine) = @_;
	my $S;
	my $proto = getprotobyname("tcp");
	my $ret = socket($S, PF_INET, SOCK_STREAM, $proto);

	if ($ret != 1) {
		print("ERROR:1:Unable to create a socket\n");
		exit(1)
	}

	my $iaddr = inet_aton($machine);
	my $port = 143;	# imap port
	my $paddr = sockaddr_in($port, $iaddr);

	my $ret = connect($S, $paddr);

	if ($ret != 1) {
		print("ERROR:2:Unable to connect to $machine:imap\n");
		return(-1);
	}
	# get the initial greeting message
	$line = msgimap_recv($S);
	return ($S);
}

# return null string if got nothing/timeout
sub
msgimap_recv
{
	my ($S) = @_;
	my ($line, $ret, $rin);

	$rin = '';
	vec($rin, fileno($S), 1) = 1;

	$ret = select($rout = $rin, undef, undef, 10.0);
	if ($ret == 1) {
		recv($S, $line, 99999, 0);
		if ($LOG) {
			print("<$line");
		}
		return($line);
	} else {
		print(STDERR "ERROR:3:Timeout on receive\n");
		return("");
	}
}

# return codes are which of the three IMAP protocol possibilities matched
# 0 is good match.
# 1 is bad match.
# 2 is ugly match.
# -1 is some kind of socket failure, probably other end disconnected.
sub
msgimap_protsend
{
	my ($S, $msg, $good, $bad, $ugly) = @_;
	my ($ret, $tmp);

	if ($LOG) {
		print(">$msg");
	}
	if (!send($S, $msg, 0)) {
		return(-1);
	}
	$ret = "";
	$rbuf = "";
	while ($ret eq "") {
		$tmp = msgimap_recv($S);
		if ($tmp eq "") {
			$ret = -1;
		}
		$rbuf = $rbuf . $tmp;
		if (index($rbuf, $good) >= 0) {
			$ret = 0;
		}
		if (index($rbuf, $bad) >= 0) {
			$ret = -2;
		}
		if (index($rbuf, $ugly) >= 0) {
			$ret = -3;
		}
	}
	if ($ret == -1) {
		print(STDERR "ERROR:6:Socket error with imap server:\n$msg");
		print(STDERR "$rbuf");
	}
	if ($ret == -2) {
		print(STDERR "ERROR:4:Command failed on imap server:\n$msg");
		print(STDERR "$rbuf");
	}
	if ($ret == -3) {
		print(STDERR "ERROR:5:Command rejected by imap server:\n$msg");
		print(STDERR "$rbuf");
	}
	return($ret);
}

return(1);
