#!/usr/bin/perl
#
# Copyright (C) 2002 Caldera International, Inc.  All Rights Reserved.
#

my $rootdn = shift @ARGV;
my $rootpw = shift @ARGV;
my $SCRIPTPATH = shift @ARGV;

my $MSG_LDAPDIR=$ENV{MSG_LDAPDIR};
my $MSG_UNITEDLINUX=$ENV{MSG_UNITEDLINUX};

#my $rootdn = 'cn=Manager,dc=my-domain,dc=com';
#my $rootpw = 'secret';
my $MSG_DIR = 'lsb-caldera.com-volution/msg';

my $ldapdbdir;
my $initdir;
my $ldiffile = '/var/tmp/msgupgrade/ldapdata.ldif';
my $domainsfile="/var/tmp/msgupgrade/domains";
my $msgconffile = "/etc/opt/$MSG_DIR/msg.conf";
my $schemafile = "$MSG_LDAPDIR/schema/msg.schema";
my $slapdconffile = "$MSG_LDAPDIR/slapd.conf";
my $ldapconffile = "$MSG_LDAPDIR/lapd.conf";

if ($MSG_UNITEDLINUX) {
	$initdir = '/etc/init.d';
	$ldapdbdir = '/var/lib/ldap';
} else {
	$initdir = '/etc/rc.d/init.d';
	$ldapdbdir = '/var/ldap/openldap-ldbm';
}

# replace Volution Messaging Server with SCOoffice Server in msg.conf file
sub fix_msgconf {
	my $buf;
	open(LDDB, $msgconffile);
	while(<LDDB>) {
		s/Volution Messaging Server/SCOoffice Server/g;
		$buf .= $_;
	}
	close LDDB;
	open(LDDB, ">$msgconffile");
	print LDDB $buf;
	close LDDB;
}

# replace Volution Messaging Server with SCOoffice Server in slapd.conf file
sub fix_slapdconf {
	my $buf;
	open(LDDB, $slapdconffile);
	while(<LDDB>) {
		s/ou=Volution Messaging Server/ou=SCOoffice Server/g;
		$buf .= $_;
	}
	close LDDB;
	open(LDDB, ">$slapdconffile");
	print LDDB $buf;
	close LDDB;
}

# replace Volution Messaging Server with SCOoffice Server in schema
sub fix_schema {
	my $buf;
	open(LDDB, $schemafile);
	while(<LDDB>) {
		s/Volution Messaging Server/SCOoffice Server/g;
		$buf .= $_;
	}
	close LDDB;
	open(LDDB, ">$schemafile");
	print LDDB $buf;
	close LDDB;
}

# reload the ldap database with the modified data
sub reload_ldapdb {
	system("ldapadd -x -c -D $rootdn -w $rootpw -f $ldiffile");
}

# remove ldap database files
sub remove_ldapdb {
	system ("rm $ldapdbdir/*");
}

# sort the ldap records so they get added top to bottom
sub sort_ldif {
	my $lddb;
	open(LDDB, "perl $SCRIPTPATH/ldifsort.pl -k dn $ldiffile |");
	while(<LDDB>) {
		$lddb .= $_;
	}
	close LDDB;
	open(LDDB, ">$ldiffile");
	print LDDB $lddb;
	close LDDB;
}

# save the names of configured domains
sub get_domains {
        my $lddb, %domain, $domainlist;
        open(LDDB, $ldiffile);
        while(<LDDB>) {
                chomp;
                if ($_ eq "") {
                        if ($domain{"objectClass"} =~ m/,msgDomain,/) {
                                $domainlist .= $domain{"ou"} . "\n";
                        }
                        %domain = ();
                        $domain{"objectClass"} = ",";
                        next;
                }
 
                ($attr, $value) = split(":", $_, 2);
		$value =~ s/^\s*//; $value =~ s/\s*$//; # remove spaces
                $domain{$attr} .= $value;
		$domain{$attr} .= "," if ($attr eq "objectClass");
        }
        close LDDB;
        open(FD, ">$domainsfile");
        print FD $domainlist;
        close FD;
}

# remove record attributes not used by ldapadd
# remove orphaned ldap entries
sub fix_ldif {
	my $lddb;
	open(LDDB, $ldiffile);
	while(<LDDB>) {
		if (/^createTimestamp/) {
			next;
		} elsif (/^calderaMsgMgrpDeliveryProgram:/) {
			next;
		} elsif (/^calderaMsgMgrpDeliveryFile:/) {
			next;
		} elsif (/^modifyTimestamp/) {
			next;
		} elsif (/^creatorsName/) {
			next;
		} elsif (/^modifiersName/) {
			next;
		} else {
			$lddb .= $_;
		}
	}
	close LDDB;
	open(LDDB, ">$ldiffile");
	print LDDB $lddb;
	close LDDB;
}

# dump the contents of the ldap database
sub get_ldif {
	my $buf, $lddb;
	open(DB, "/usr/sbin/slapcat |");
	while(<DB>) {
		if (/^ /) {
			chomp $buf if ($buf);
			$buf .= substr($_, 1);
		} elsif ($buf) {
			$buf =~ s/Volution Messaging Server/SCOoffice Server/g;
			$lddb .= $buf;
			undef($buf);
			$buf = $_;
		} else {
			$buf = $_;
		}
	}
	close DB;
	system("mkdir -p `dirname $ldiffile`");
	open(LDDB, ">$ldiffile");
	print LDDB $lddb;
	close LDDB;
}

system ("$initdir/ldap stop");
sleep 2;
system ("killall -15 slapd");
sleep 2;
system ("killall -9 slapd");
sleep 2;
get_ldif;
fix_ldif;
sort_ldif;
remove_ldapdb;
fix_schema;
fix_slapdconf;
fix_msgconf;
get_domains;
system ("cd /; $initdir/ldap start");
sleep 2;
reload_ldapdb;
#unlink $ldiffile;

system ("$initdir/ldap stop");
sleep 2;
system ("killall -15 slapd");
sleep 2;
system ("killall -9 slapd");
sleep 2;

exit 0;
