<?php
/*
 * Copyright (C) 2002 Caldera International, Inc.  All Rights Reserved.
 */

class LicenseList extends ObjectTable {
	
	var $utotal;      // total number of licensed users
	var $uinuse;      // number of user licenses used
	var $masterSerial; // the serial number that enables the product

	function LicenseList($varArray, $id) {

		$this->list = '';
		$cmd = new Command('msglicenselistall');
		$cmd->run();
		$lines = $cmd->getOutput();

		unset($this->list);
		$this->utotal = 0;

		// Populate the list 
		if ($lines != '') foreach ($lines as $line) {

			$line = trim($line);

			if ($line == '') {
				if ($license != '') {
					$this->utotal += $license->getUsers();
					$this->list[] = $license;
					$license = '';
				}
				continue;
			}

			if ($license == '' && $line != '') {
				$license = new license();
			}

			list($attr, $value) = split(':', $line, 2);
			$license->setAttr($attr, $value);
		}

		// Get the master serial number
		// Get the count of users being used
		$tmp = new MasterLicense();
		if ($tmp) {
			$this->masterSerial = $tmp->getSerialNumber();
			$this->uinuse = $tmp->getUsersInUse();
		}
		unset($tmp);

		// Setup the table
		if (Table::isActive($varArray, $id)) {

			$table = Table::getActive();
			$table->setParams($varArray, $this->count());

		} else {

			$table = new Table($id);

			$name = getIntlMsg('MSG_LICENSE_HEADING_SERIAL');
			$table->addDataColumn($name, Column::ASCEND(),
				'serial');

			$name = getIntlMsg('MSG_LICENSE_HEADING_DESC');
			$table->addDataColumn($name, Column::ASCEND(),
				'description');

			$name = getIntlMsg('MSG_LICENSE_HEADING_USERS');
			$table->addDataColumn($name, Column::ASCEND(),
				'users');

			$name = getIntlMsg('MSG_LICENSE_HEADING_DELETE');
			$table->addDisplayColumn($name,
				"return Go('licensedelete_action.php?serial=%%ROWID%%')");

			$table->setParams($HTTP_GET_VARS, $this->count());
			$table->rows = $GLOBALS['RULES_TABLE_ROWS'];
			$table->setSortedColumn($table->firstSortedColumn());

		}

		// Apply sorting
		$attr = $table->getSortAttrName($table->getSortedColumn());
		$dir  = $table->getColumnSortDir($table->getSortedColumn());
		$this->sort(License::comparisonFunctionName($attr ,$dir));

		// Save table
		$this->table = $table;
		$this->table->save();
	}

	function getListValue($index, $col = '') {
		global $session_lang;

		$lic = $this->list[$index];

		// if no $col then return the row value (unique id)
		if ($col === '') {
			return $lic->getSerial();
		}

		switch ($col) {
			case '0':
			case 'serial':
				$retval = $lic->getSerial();
				break;
			case '1':
			case 'description':
				$retval = $lic->getDescription();
				break;
			case '2':
			case 'users':
				$retval = $lic->getUsers();
				break;
			case '3' :
				if ($this->masterSerial == $lic->getSerial()) {
					$retval = '';
				} else {
					$retval = $this->table->getTemplate(
						'deleteIcon');
					$retval = ereg_replace('%%LANG%%', 
						$session_lang, $retval);
					$retval = stripslashes($retval);
				}
				break;
			default:
				$retval = '';
				break;
		}

		return $retval;
	}

	function getULicenseTotal() {
		return $this->utotal;
	}

	function getULicenseInUse() {
		return ($this->uinuse > 0) ?  $this->uinuse : 0;
	}

	function getULicenseAvailable() {
		return ($this->utotal > $this->uinuse) ? 
			$this->utotal - $this->uinuse : 0;
	}
}


?>
