<?php
/*
 * Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
 */

class MailAddress {
	var $mail;
	var $type;

	function MailAddress() {
		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 0:
			$this->mail = '';
			break;
		  case 1:
			$this->_mailAddress(func_get_arg(0));
			break;
		  default:
			$this->mail = '';
			break;
		}
	}

	function _mailAddress($address) {
		global $session_domain;

		$pos = strpos($address, '@');
		if ($pos === false) {
			$address = $address . '@' . $session_domain;
		}
		$this->mail = $address;
		$this->type = $this->determineType($address);

		if (! $this->type) {
			$this = '';
		}
	}

	function stripLocalDomain() {
		global $session_domain;

		$mail = $this->mail;
		$domain = $session_domain;

		$numArgs = func_num_args();

		switch ($numArgs) {
		  case 1:
			$mail = func_get_arg(0);
			break;
		  case 1:
			$mail = func_get_arg(0);
			$domain = func_get_arg(1);
			break;
		  default:
			break;
		}

		$pos = strpos($mail, '@' . $domain);
		if ($pos !== false) {
			$mail = substr($mail, 0, $pos);
		}

		return $mail;
	}

	function isValid($address) {
		$ret = true;
		$debug = false;

		// characters allowed in name: 0-9a-Z-._ in host: 0-9a-Z-. 
		// in between: @
		if (!ereg('^[0-9a-zA-Z\.\_\-]+\@[0-9a-zA-Z\.\-]+$', $address)) {
			$ret = false;
			if ($debug) guilog("$address failed1");
		}
		
		// must start or end with alpha or num
		if (ereg('^[^0-9a-zA-Z]|[^0-9a-zA-Z]$', $address)) {
			$ret = false;
			if ($debug) guilog("$address failed2");
		}

		// name must end with alpha or num
		if (!ereg('([0-9a-zA-Z]{1})\@.', $address)) {
			$ret = false;
			if ($debug) guilog("$address failed3");
		}

		// host must start with alpha or num
		if (!ereg('.\@([0-9a-zA-Z]{1})', $address)) {
			$ret = false;
			if ($debug) guilog("$address failed4");
		}
		
		// pair .- or -. or -- or .. not allowed		
		if (ereg('.\.\-.|.\-\..|.\.\..|.\-\-.', $address)) {
			$ret = false;
			if ($debug) guilog("$address failed5");
		}

		// pair ._ or -_ or _. or _- or __ not allowed
		if (ereg('.\.\_.|.\-\_.|.\_\..|.\_\-.|.\_\_.', $address)) {
			$ret = false;
			if ($debug) guilog("$address failed6");
		}

		// host must end with '.' plus 2 or 3 alpha for 
		// TopLevelDomain (MUST be modified in future!)
		if (!ereg('\.([a-zA-Z]{2,3})$', $address)) {
			$ret = false;
			if ($debug) guilog("$address failed7");
		}

		if ($debug && ! $ret) guilog("$address returns $ret");
		return $ret;
	}

	function determineType($address) {
		global $session_domain;

		$remote = false;
		$pos = strpos($address, '@');
		if ($pos === false) {
			$remote = false;
			$address = $address . '@' . $session_domain;
		} elseif (substr($address, $pos + 1) == $session_domain) {
			$remote = false;
		} else {
			$remote = true;
		}

		if (! $this->isValid($address)) {
			return false;
		}

		if ($remote) {
			return 'remote';
		}

		// is it a user mail address
		$tmp = new User($address);
		unset($GLOBALS['LAST_ERROR']);
		if ($tmp && $tmp->mail == $address) {
			return 'user';
		} 

		// is it an alias mail address
		$tmp = new Alias($address);
		unset($GLOBALS['LAST_ERROR']);
		if ($tmp && $tmp->mail == $address) {
			return 'alias';
		}
	}
}
?>
