<?php
/*
 * Copyright (C) 2002 Caldera International, Inc.  All Rights Reserved.
 */

class RuleList extends ObjectTable {
	
	var $user;

	function RuleList($userObj, $varArray, $id) {


		if ($userObj == '') {
			$this = '';
			return;
		} else {
			$this->user = $userObj;
		}

		if ($this->user->spamallow) 
		foreach ($this->user->spamallow as $s) {
			$this->list[] = new RuleFactory('allowfrom', $s);
		}

		if ($this->user->spamdeny) 
		foreach ($this->user->spamdeny as $s) {
			$this->list[] = new RuleFactory('denyfrom', $s);
		}

		// Setup the table
		if (Table::isActive($varArray, $id)) {

			$table = Table::getActive();
			$table->setParams($varArray, $this->count());

		} else {

			$table = new Table($id);

			$name = getIntlMsg('MSG_RULES_HEADING_STRING');
			$table->addDataColumn($name, Column::ASCEND(),
				'string');

			$name = getIntlMsg('MSG_RULES_HEADING_RULETYPE');
			$table->addDataColumn($name, Column::ASCEND(),
				'className');

			$name = getIntlMsg('MSG_RULES_HEADING_REMOVE');
			$table->addDisplayColumn($name,
				"return Go('ruleremove_action.php?" .
				$this->alias->mail . 
				"%%ROWID%%')");

			$table->setParams($HTTP_GET_VARS, $this->count());
			$table->rows = $GLOBALS['RULES_TABLE_ROWS'];
			$table->setSortedColumn($table->firstSortedColumn());

		}

		// Apply sorting
		$attr = $table->getSortAttrName($table->getSortedColumn());
		$dir  = $table->getColumnSortDir($table->getSortedColumn());
		$this->sort(Rule::comparisonFunctionName($attr ,$dir));

		// Save table
		$this->table = $table;
		$this->table->save();
	}

	function getListValue($index, $col = '') {
		global $session_lang;

		$rule = $this->list[$index];
		$class = get_class($rule);

		// if no $col then return the row value (unique id)
		if ($col === '') {
			$retval  = 'id=' . rawurlencode($rule->getId());
			$retval .= '&class=' . rawurlencode($rule->getClass());
			return $retval;
		}

		switch ($col) {
			case '0':
			case 'string':
				$retval = $rule->getId();
				break;
			case '1':
			case 'className':
				$retval = $rule->getClassName();
				break;
			case '2' :
				$retval = $this->table->getTemplate('deleteIcon');
				$retval = ereg_replace('%%LANG%%', $session_lang, $retval);
				$retval = stripslashes($retval);
				break;
			default:
				$retval = '';
				break;
		}

		return $retval;
	}

	function getNav() {
		global $_SELF, $_SERIAL;
	
		$current = (int)($this->table->offset / $this->table->rows) + 1;

		$total = (int)($this->count() / $this->table->rows);
		if ($this->count() % $this->table->rows > 0) {
			$total++;
		} elseif ($this->count() == 0) {
			$total = 1;
		}
	
		$retval = '<div class="tablenav">';
		$retval .= getIntlMsg('MSG_TABLE_CURRENT_PAGE',
			   $current, $total);
		if ($total == 1) {
			return $retval;
		}
 		$retval .= ' ' . getIntlMsg('MSG_TABLE_CURRENT_SEP') . ' ';
		if ($current > 1) {
			$offset = ($current - 2) * $this->table->rows;
			$href = $_SELF .
				'?mail='   . $this->alias->mail .
				'&offset=' .  $offset .
				'&serial=' . $_SERIAL;
			$retval .= ' <a href="' . $href . 
				 '" class="tablenav">' .
				getIntlMsg('MSG_TABLE_PAGE_PREV') . '</a> ';
		}
		for ($i = 1; $i <= $total; $i++) {
			$offset = ($i - 1) * $this->table->rows;
			$href = $_SELF .  
				'?mail=' . $this->alias->mail .
				'&offset=' . $offset .
				'&serial=' . $_SERIAL;
			if ($current != $i) {
				$retval .= ' <a href="' . $href . 
					'" class="tablenav">' . $i . '</a> ';
			} else {
				$retval .= " $i ";
			}
		}
		if ($current < $total) {
			$offset = ($current) * $this->table->rows;
			$href = $_SELF .  
				'?mail=' . $this->alias->mail . 
				'&offset=' . $offset .
				'&serial=' . $_SERIAL;
			$retval .= ' <a href="' . $href . 
				'" class="tablenav">' . 
				getIntlMsg('MSG_TABLE_PAGE_NEXT') . '</a> ';
		}
		$retval .= '</div>' . "\n";

		return $retval;

	}

}

class RuleFactory {
	function RuleFactory($type, $id) {
		switch ($type) {
			case 'allowfrom':
				$this = new AllowFromRule($id);
				break;
			case 'denyfrom':
				$this = new DenyFromRule($id);
				break;
			default:
				$this = '';
				break;
		}
	}
}

class Rule {
	var $id;
	var $class;
	var $className;

	function Rule($id) {
		$this->id = $id;
		$this->class = 'generic';
		$this->className = getIntlMsg('MSG_USERS_RULE_CLASS_GENERIC');
	}

	function getId() {
		return $this->id;
	}

	function getClass() {
		return $this->class;
	}

	function getClassName() {
		return $this->className;
	}

	function isAllow() {
		return false;
	}

	function isDeny() {
		return false;
	}

	function comparisonFunctionName($field, $direction) {
		switch($field) {
			case 'id':
				$cmpFunction = '_rule_byId';
				break;
			case 'class':
				$cmpFunction = '_rule_byClass';
				break;
			case 'className':
				$cmpFunction = '_rule_byClassName';
				break;
			default:
				$cmpFunction = '_rule_byId';
				break;
		}

		switch($direction) {
			case 'a':
				$cmpFunction .= 'Ascend';
				break;
			case 'd':
				$cmpFunction .= 'Decend';
				break;
			default:
				$cmpFunction .= 'Acend';
				break;
		}
		return $cmpFunction;
	}

}

/*
 * Ideally these comparison functions would be encapsulated in the 
 * User class as class methods. Can't work out how to pass a class
 * method to the usort() function yet.
 */
function _rule_byIdAscend($left, $right) {
	return(strcasecmp($left->id,$right->id));
}

function _rule_byIdDecend($left, $right) {
	return(-1 * (_rule_byIdAscend($left, $right)));
}

function _rule_byClassAscend($left, $right) {
	return(strcasecmp($left->class,$right->class));
}

function _rule_byClassDecend($left, $right) {
	return(-1 * (_rule_byClassAscend($left, $right)));
}

function _rule_byClassNameAscend($left, $right) {
	return(strcasecmp($left->className,$right->className));
}

function _rule_byClassNameDecend($left, $right) {
	return(-1 * (_rule_byClassNameAscend($left, $right)));
}

class AllowFromRule extends Rule {
	function AllowFromRule($id) {
		$this->Rule($id);
		$this->class = 'allowfrom';
		$this->className = getIntlMsg('MSG_USERS_RULE_CLASS_ALLOW');
	}

	function isAllow() {
		return true;
	}
}

class DenyFromRule extends Rule {
	function DenyFromRule($id) {
		$this->Rule($id);
		$this->class = 'denyfrom';
		$this->className = getIntlMsg('MSG_USERS_RULE_CLASS_DENY');
	}

	function isDeny() {
		return true;
	}
}

?>
