#!/usr/bin/perl
#
# Copyright (c) 2001-2002 Caldera International, Inc. All Rights Reserved.
#

BEGIN {
	unshift(@INC, "/opt/lsb-sco.com/msg/bin");
}

use msglib;


#-----------------------------------------------------------------------------
# Three states for parsing.
# NOTFOUND  - we haven't found what we're looking for
# FOUND     - we've found what we're looking for and we're still finding it
# INSTALLED - we've found what we're looking for and we're done with modifying
#-----------------------------------------------------------------------------
my $NOTFOUND=0;
my $FOUND=1;
my $INSTALLED=2;

#-----------------------------------------------------------------------------
# Three variables associated with httpd.conf directives
#  	inloadmod - the LoadModule directive
#	inaddmod  - the AddModule directive
#	msgmod    - the configuration for MSG specific directories
#-----------------------------------------------------------------------------
my $inloadmod=$NOTFOUND;
my $inaddmod=$NOTFOUND;
my $insslhost=$NOTFOUND;
my $msgmod=$NOTFOUND;
my $davlock=$NOTFOUND;

my $modsrc="/opt/lsb-sco.com/msg/bin/mod_auth_ldap.so";
my $moddest="/usr/lib/apache/mod_auth_ldap.so";
my $htconf="/etc/httpd/httpd.conf";
$hostname=`/bin/hostname --fqdn`;
delete($ENV{ROOT});

$vartmpdir='/var/opt/lsb-sco.com/msg/tmp';
if ($UNITEDLINUX) {
	$initdir="/etc/init.d";
	$ftpdir="/srv/ftp";
	$httplib="/usr/lib/apache";
	$httpuser="wwwrun";
	$calendar_dir='/srv/ftp/pub/calendar';
} else {
	$initdir="/etc/rc.d/init.d";
	$ftpdir="/home/ftp";
	$httplib="lib/apache";
	$httpuser="httpd";
	$calendar_dir='/home/httpd/html/calendar';
}

#-----------------------------------------------------------------------------
# print_htconf - prints most of the platform-specific htconf info
#-----------------------------------------------------------------------------
sub print_htconf {
print NEW '# START SCOoffice Mail Server CONFIGURATION - DO NOT MODIFY' . "\n";
print NEW "DAVLockDB $vartmpdir/DAVLock\n";
print NEW 'Alias /msg "/opt/lsb-sco.com/msg/webgui/"
<Directory "/opt/lsb-sco.com/msg/webgui">
        Options Indexes MultiViews ExecCGI
        AllowOverride None Limit FileInfo Options
        Order allow,deny
        Allow from all
</Directory>
Alias /msghelp "/usr/share/doc/packages//scomsg"
<Directory "/usr/share/doc/packages/scomsg">
        Options Indexes MultiViews
        AllowOverride None
        Order allow,deny
        Allow from all
</Directory>
';

print NEW "Alias /calendar \"$calendar_dir/\"
Alias /pub/calendar \"$calendar_dir/\"
<Directory \"$calendar_dir\">
";

print NEW '	Dav on
	Options Indexes MultiViews
	order allow,deny
	allow from all
	AuthName "Calendar"
	AuthType Basic
';

if ($UNITEDLINUX) {
	print NEW '	AuthAuthoritative off
        AuthLDAPAuthoritative on
';
}

print NEW "	LDAP_Server $ldaphost
	LDAP_Port   $ldapport
	Base_DN     \"$ldapsuffix\"
	Bind_DN     \"$msgdn\"
	Bind_Pass   $msgpw
	UID_Attr    uid
	require valid-user
";
print NEW '
</Directory>
# END SCOoffice Mail Server CONFIGURATION
';
}

# Install the mod_auth_ldap Apache module
if ( -f $moddest ) 
{
	print "mod_pam_ldap already installed\n";
}
else
{
	system("cp -f $modsrc $moddest");
}


# Use parse_conf to load the msg.conf information.
# Setting msgadminpw to something causes parse_conf to not 
# prompt for a password (which we don't want).
$msgadminpw="XXX";
parse_conf "noauth";

# Derive the default LDAP port from the global variables returned by
# parse_conf.
$ldapport=389;
$_ = $ldaphost;
if (/.*:.*/) {
	($ldaphost, $ldapport) = split(":");
}

# Begin modifying httpd.conf
open(OLD, "$htconf") || die "$htconf:$!";
open(NEW, ">$htconf.bak") || die "$htconf.bak:$!";

# First check to see if it is already modified
# If it is, bail out.
while(<OLD>) 
{
	if (/.*mod_auth_ldap.*/ && ! /^#/ ) 
	{
		print STDERR "mod_auth_ldap already configured\n";
		$inloadmod=$INSTALLED;
		$inaddmod=$INSTALLED;
	}
}

seek(OLD, 0, 0);
while(<OLD>) 
{
	if (/^DavLockDB.*/) {
		# get rid of old DavLockDB if any
		next;
	}

	if (/^LoadModule.*/) 
	{
		if ( $inloadmod == $NOTFOUND ) {
			$inloadmod=$FOUND;
		}
	}
	elsif (/^AddModule.*/) 
	{
		if ( $inaddmod == $NOTFOUND ) {
			$inaddmod=$FOUND;
		}
	}
	elsif (/^# END SCOoffice Mail Server CONFIGURATION/) {
		$msgmod=$INSTALLED;
		next;
	}
	elsif (/^# START SCOoffice Mail Server CONFIGURATION - DO NOT MODIFY/) 
	{

		print_htconf;
		$msgmod=$FOUND;
		next;
	}
	elsif (/^\<VirtualHost _default_:443\>/)
	{
		if ( $insslhost == $NOTFOUND ) {
			$insslhost = $FOUND;
		}
	}
	else 
	{
		if ( $insslhost == $FOUND) {
			if (/^ServerName.*/) {
				print NEW "ServerName $hostname\n";
				next;
			} elsif (/^ServerAdmin.*/) {
				print NEW "ServerAdmin root\@$hostname\n";
				$insslhost = $INSTALLED;
				next;
			}
		}

		if ( $inloadmod == $FOUND ) {
			print NEW "LoadModule ldap_auth_module   $httplib/mod_auth_ldap.so\n";
			$inloadmod=$INSTALLED;
		}
		if ( $inaddmod == $FOUND ) {
			print NEW "AddModule mod_auth_ldap.c\n";
			$inaddmod=$INSTALLED;
		}
		if ( $msgmod == $FOUND ) {
			next;
		}
	}
	print NEW $_;
}
if ( $msgmod != $INSTALLED ) {
	print_htconf;
}
close(OLD);
close(NEW);
rename("$htconf.bak", "$htconf");

# enable mod_dav on UnitedLinux
if ($UNITEDLINUX) {

	#system('cd /usr/share/doc/packages/mod_ssl; ./certificate.sh');

	my $change = 0;
	my $file = "/etc/sysconfig/apache";
	open(OLD, "$file") || die "$file:$!";
	open(NEW, ">$file.tmp") || die "$file.tmp:$!";
	while(<OLD>) {
        	if (/.*HTTPD_SEC_MOD_DAV=no.*/ && ! /^#/ ) {
			print NEW "HTTPD_SEC_MOD_DAV=yes\n";
			$change = 1;
		} elsif (/.*HTTPD_SEC_MOD_SSL=no.*/ && ! /^#/ ) {
			print NEW "HTTPD_SEC_MOD_SSL=yes\n";
			$change = 1;
		} else {
			print NEW $_;
		}
        }
	close(OLD);
	close(NEW);
	if ($change) {
		rename("$file.tmp", "$file");
		system("/sbin/SuSEconfig --module apache");
	} else {
		unlink("$file.tmp");
	}
}


# Reload httpd to suck in the changes.
system("$initdir/httpd", "stop");
system("$initdir/httpd", "start");


# Configure the free/busy calendar directory for Web Folders 
# support.
system("/bin/mkdir -p $ftpdir/pub/calendar/.DAV");
system("/bin/chmod 770 $ftpdir/pub/calendar/.DAV");

# Resolve permission conflicts between ftp and httpd.  Anon FTP
# will now share the httpd uid as will all anonymous files.
$ftpid = getpwnam($httpuser);
system("/usr/sbin/usermod -u $ftpid -o ftp");
system("/bin/chown -R ftp $ftpdir/pub");
exit(0);
